#ifndef TEMPLATE_CACHE_H
#define TEMPLATE_CACHE_H

/*
 * template_cache.h
 * Provides a attribute value cache for template entries.
 * The cache is implemented as sparsely populated matrix of dimension
 * <number of schema attributes> X <maximum number of cacheable entries>.
 *
 * Cache indices are computed by looking up attribute names and entry
 * identifiers (i.e. 'cn' or 'uid' attribute values) in symbol tables,
 * which keep unique integer values for each registered string.
 *
 * Thus, access to saved values in constant time is granted.
 *
 * Author: H. Schulz <helmut@thalion-graphics.de>
 * (C) Thalion Graphic Solutions eK. 2007, all rights reserved
 *
 * Developed for Hewlett-Packard GmbH Germany
 * Contact: Bernd Schumacher <bernd.schumacher@hp.com>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted only as authorized by the
 * GNU Public License 2.0.
 *
 * A copy of this license is available at
 * <http://www.fsf.org/licensing/licenses/gpl.txt>
 */

#include "template_symtab.h"

typedef struct template_cache template_cache_t;

#ifdef TEMPLATE_CACHE_STANDALONETEST
typedef char * template_cachedval_t;
#else
#include "portable.h"
#include "slap.h"
typedef BerVarray template_cachedval_t;
#endif

template_cache_t *template_cache_create(int nattr,int ncos);
template_cache_t *template_cache_init(template_cache_t *cache,int nattr,int ncos);
template_cachedval_t template_cache_get(template_cache_t *cache,int att,const char *cos,int copy,void *ctx);
int template_cache_put(template_cache_t *cache,int att,const char *cos,template_cachedval_t val);
void template_cache_clear(template_cache_t *cache);
void template_cache_deinit(template_cache_t *cache);
void template_cache_free(template_cache_t *cache);

#endif
