#ifndef TEMPLATE_MALLOC_H
#define TEMPLATE_MALLOC_H

/*
 * template_malloc.h
 * Bunch of macros to decouple memory management from openldap,
 * to allow standalone testing for cache and symtab.
 *
 * Author: H. Schulz <helmut@thalion-graphics.de>
 * (C) Thalion Graphic Solutions eK. 2007, all rights reserved
 *
 * Developed for Hewlett-Packard GmbH Germany
 * Contact: Bernd Schumacher <bernd.schumacher@hp.com>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted only as authorized by the
 * GNU Public License 2.0.
 *
 * A copy of this license is available at
 * <http://www.fsf.org/licensing/licenses/gpl.txt>
 */

/* On AMD64 (only? Need tests on other 64bit architectures), ch_malloc() fails, when invoked simultaneously from different threads.
   As a workaround, the offending calls are replaced by stdlib memory mgmt functions.
   This is ugly, and at this point I don't know whether it's actually a ch_malloc() bug, or delayed consequence of something nasty
   within template overlay code. */

#ifdef __amd64__
#define TEMPLATE_AMD64_HACK
#endif

#if defined(TEMPLATE_CACHE_STANDALONETEST) ||\
    defined(TEMPLATE_SYMTAB_STANDALONETEST) ||\
    defined(TEMPLATE_AMD64_HACK)
/*
 * To allow standalone testing:
 */
#define TEMPLATE_MALLOC(size) malloc(size)
#define TEMPLATE_FREE(p) free(p)
#define TEMPLATE_STRDUP(p) strdup(p)

#else

#define TEMPLATE_MALLOC(size) ch_malloc(size)
#define TEMPLATE_FREE(p) ch_free(p)
#define TEMPLATE_STRDUP(p) ch_strdup(p)

#endif

#endif
